package app.simple.inure.ui.panels

import android.content.SharedPreferences
import android.content.pm.PackageInfo
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.view.doOnPreDraw
import androidx.lifecycle.ViewModelProvider
import app.simple.inure.R
import app.simple.inure.adapters.ui.AdapterBatch
import app.simple.inure.constants.BundleConstants
import app.simple.inure.decorations.corners.DynamicCornerLinearLayout
import app.simple.inure.decorations.overscroll.CustomVerticalRecyclerView
import app.simple.inure.decorations.ripple.DynamicRippleImageButton
import app.simple.inure.decorations.theme.ThemeDivider
import app.simple.inure.dialogs.app.Sure
import app.simple.inure.dialogs.batch.BatchMenu
import app.simple.inure.dialogs.batch.BatchUninstaller
import app.simple.inure.dialogs.batch.DialogBatchSelectedApps
import app.simple.inure.dialogs.menus.AppsMenu
import app.simple.inure.extensions.fragments.ScopedFragment
import app.simple.inure.interfaces.adapters.AdapterCallbacks
import app.simple.inure.interfaces.fragments.SureCallbacks
import app.simple.inure.models.BatchPackageInfo
import app.simple.inure.popups.batch.PopupBatchAppsCategory
import app.simple.inure.popups.batch.PopupBatchSortingStyle
import app.simple.inure.preferences.BatchPreferences
import app.simple.inure.ui.actions.BatchExtract
import app.simple.inure.util.ViewUtils.gone
import app.simple.inure.util.ViewUtils.visible
import app.simple.inure.viewmodels.panels.BatchViewModel

class Batch : ScopedFragment() {

    private lateinit var recyclerView: CustomVerticalRecyclerView
    private lateinit var divider: ThemeDivider
    private lateinit var batchMenuContainer: DynamicCornerLinearLayout
    private lateinit var delete: DynamicRippleImageButton
    private lateinit var send: DynamicRippleImageButton
    private lateinit var extract: DynamicRippleImageButton
    private lateinit var checklist: DynamicRippleImageButton

    private var adapterBatch: AdapterBatch? = null
    private lateinit var batchViewModel: BatchViewModel

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.fragment_batch, container, false)

        recyclerView = view.findViewById(R.id.batch_recycler_view)
        divider = view.findViewById(R.id.divider)
        batchMenuContainer = view.findViewById(R.id.batch_menu_container)
        delete = view.findViewById(R.id.delete)
        send = view.findViewById(R.id.send)
        extract = view.findViewById(R.id.extract)
        checklist = view.findViewById(R.id.menu)

        batchViewModel = ViewModelProvider(requireActivity())[BatchViewModel::class.java]

        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        showLoader()

        batchViewModel.getAppData().observe(viewLifecycleOwner) {
            adapterBatch = AdapterBatch(it)
            batchMenuState(it)
            hideLoader()

            adapterBatch?.setOnItemClickListener(object : AdapterCallbacks {
                override fun onAppClicked(packageInfo: PackageInfo, icon: ImageView) {
                    openFragmentArc(AppInfo.newInstance(packageInfo, icon.transitionName), icon, "app_info")
                }

                override fun onAppLongPressed(packageInfo: PackageInfo, icon: ImageView) {
                    AppsMenu.newInstance(packageInfo)
                        .show(childFragmentManager, "apps_menu")
                }

                override fun onSearchPressed(view: View) {
                    openFragmentSlide(Search.newInstance(true), "search")
                }

                override fun onSettingsPressed(view: View) {
                    BatchMenu.newInstance()
                        .show(childFragmentManager, "batch_menu")
                }

                override fun onBatchChanged(batchPackageInfo: BatchPackageInfo) {
                    batchViewModel.updateBatchItem(batchPackageInfo)
                    batchMenuState(adapterBatch!!.getCurrentAppsList())
                }

                override fun onSortPressed(view: View) {
                    PopupBatchSortingStyle(view)
                }

                override fun onFilterPressed(view: View) {
                    PopupBatchAppsCategory(view)
                }
            })

            recyclerView.adapter = adapterBatch

            (view.parent as? ViewGroup)?.doOnPreDraw {
                startPostponedEnterTransition()
            }
        }

        delete.setOnClickListener {
            val p0 = Sure.newInstance()
            p0.setOnSureCallbackListener(object : SureCallbacks {
                override fun onSure() {
                    BatchUninstaller.newInstance(adapterBatch?.getCurrentAppsList()!!)
                        .show(childFragmentManager, "batch_extract")
                }
            })
            p0.show(childFragmentManager, "sure")
        }

        send.setOnClickListener {

        }

        extract.setOnClickListener {
            val p0 = Sure.newInstance()
            p0.setOnSureCallbackListener(object : SureCallbacks {
                override fun onSure() {
                    BatchExtract.newInstance(adapterBatch?.getCurrentAppsList()!!)
                        .show(childFragmentManager, "batch_extract")
                }
            })
            p0.show(childFragmentManager, "sure")
        }

        checklist.setOnClickListener {
            val d = DialogBatchSelectedApps.newInstance(adapterBatch!!.getCurrentAppsList())

            d.setOnBatchSelectedAppsCallbacks(object : DialogBatchSelectedApps.Companion.BatchSelectedAppsCallbacks {
                override fun onBatchChanged(batchPackageInfo: BatchPackageInfo) {
                    adapterBatch?.updateBatchItem(batchPackageInfo)
                }
            })

            d.show(childFragmentManager, "batch_selected_apps")
        }
    }

    private fun batchMenuState(arrayList: ArrayList<BatchPackageInfo>) {
        for (batch in arrayList) {
            if (batch.isSelected) {
                batchMenuContainer.visible(false)
                divider.visible(animate = false)
                return
            }
        }

        batchMenuContainer.gone()
        divider.gone()
    }

    override fun onSharedPreferenceChanged(sharedPreferences: SharedPreferences?, key: String?) {
        when (key) {
            BatchPreferences.moveSelectionTop -> {
                adapterBatch?.moveSelectedItemsToTheTop()
            }
            BatchPreferences.highlightSelected -> {
                adapterBatch?.updateSelectionsHighlights(BatchPreferences.isSelectedBatchHighlighted())
            }
            BatchPreferences.isSortingReversed,
            BatchPreferences.listAppsCategory,
            BatchPreferences.sortStyle -> {
                batchViewModel.refresh()
            }
        }
    }

    companion object {
        fun newInstance(loading: Boolean = false): Batch {
            val args = Bundle()
            val fragment = Batch()
            args.putBoolean(BundleConstants.loading, loading)
            fragment.arguments = args
            return fragment
        }
    }
}